package com.cescit.integrity;

import android.content.res.AssetManager;
import android.util.Base64;
import android.app.Activity;
import android.content.Context;

import org.json.JSONObject;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;


class AssetsIntegrity {

    private static final String MESSAGE_DIGEST_ALGORITHM = "SHA-256";

    private static final Map<String, String> hashList = Collections.unmodifiableMap(
        new HashMap<String, String>()
    );

    public static JSONObject check(Context context) throws Exception {
        AssetManager assets = context.getAssets();
        for (Map.Entry<String, String> entry : hashList.entrySet()) {
            // byte[] fileNameDecode = Base64.decode(entry.getKey(), 0);
            // String fileName = new String(fileNameDecode, StandardCharsets.UTF_8);
            // Log.d("AntiTampering", fileName + " -> " + entry.getValue());
            String filePath = entry.getKey();
            InputStream file = assets.open(filePath);
            String hash = getFileHash(file);
            if (entry.getValue() == null || !entry.getValue().equals(hash)) {
                throw new Exception("Content of " + filePath + " has been tampered");
            }
        }
        JSONObject result = new JSONObject();
        result.put("count", hashList.size());
        return result;
    }

    // 获取Res文件对应的hash值构造的String
    public static String getHashString(Context context) throws Exception{
        AssetManager assets = context.getAssets();
        String str = "";
        // 遍历对比文件hash
        for (Map.Entry<String, String> entry : hashList.entrySet()) {
            String fileName = entry.getKey();
            String presetHash = entry.getValue();
            if (!presetHash.equals("")) {
                str += "put(\"" + fileName + "\", \"" + presetHash + "\");";
            }
        }
        // 用默认字符编码解码字符串。
        byte[] bs = str.getBytes();
        str = new String(bs, StandardCharsets.UTF_8);
        return str;
    }

    private static String getFileHash(InputStream file) throws IOException, NoSuchAlgorithmException {
        byte[] buffer = new byte[1024]; // The buffer to read the file
      MessageDigest digest = MessageDigest.getInstance(MESSAGE_DIGEST_ALGORITHM);
        int numRead = 0; // Record how many bytes have been read
        while (numRead != -1) {
          numRead = file.read(buffer);
          if (numRead > 0)
            digest.update(buffer, 0, numRead); // Update the digest
        }
        byte [] sha1Bytes = digest.digest();
        StringBuffer hexString = new StringBuffer();
        for (int i = 0; i < sha1Bytes.length; i++) {
            hexString.append(Integer.toString(( sha1Bytes[i] & 0xff) + 0x100, 16).substring(1));
        }
        // Log.d("AntiTampering", String(hexString));
        return new String(hexString);
    }

}
